/*
    ClipGrab³
    Copyright (C) Philipp Schmieder
    http://clipgrab.de
    feedback [at] clipgrab [dot] de

    This file is part of ClipGrab.
    ClipGrab is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    ClipGrab is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ClipGrab.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "video.h"

video::video()
{
    this->_networkAccessManager = new QNetworkAccessManager(this);
    _treeItem = NULL;
}

bool video::supportsSearch()
{
    if (_supportsSearch == true)
    {
        return true;
    }
    else
    {
        return false;
    }
}

QString video::getName()
{
    return _name;
}

QIcon* video::getIcon()
{
    if (_icon)

    {
        return _icon;
    }
    else
    {
        return new QIcon(QPixmap(":/img/icon.png"));
    }
}

bool video::compatibleWithUrl(QString url)
{
    for (int i = 0; i < this->_urlRegExp.size(); ++i)
    {
        if (this->_urlRegExp.at(i).indexIn(url)!=-1)
        {
            return true;
        }
    }

    return false;

}

bool video::setUrl(QString url)
{
    if (_url.isEmpty())
    {
        this->_url = QUrl(url);
        if (_url.isValid())
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

void video::analyse()
{
    tempFile = new QTemporaryFile(QDir::tempPath() + "/clipgrab-info--XXXXXX");
    _finished = false;
    if (this->_url.isValid())
    {
        _step = 1;
        _redirectLevel = 0;
        QNetworkRequest request;
        request.setUrl(QUrl(this->_url));
        request.setRawHeader("User-Agent", "Mozilla/5.0 (compatible; Konqueror/4.2; Linux) KHTML/4.1.4 (like Gecko) SUSE");
        _currentNetworkReply = this->_networkAccessManager->get(request);
        this->_redirectLevel = 0;
        connect(_currentNetworkReply, SIGNAL(readyRead()), this, SLOT(dataHandler()));
        connect(this->_networkAccessManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(handleNetworkReply(QNetworkReply*)));
        connect(this, SIGNAL(analysingFinished()), this, SLOT(slotAnalysingFinished()));
    }
    else
    {
        emit error("Url is not valid");
    }
}

void video::handleNetworkReply(QNetworkReply* networkReply)
{
    if (!networkReply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl().isEmpty())
    {
        if (this->_redirectLevel<16)
        {
            this->_redirectLevel++;
            QUrl _url = networkReply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
            qDebug() << "Following Redirect:" << _url << _redirectLevel;
            networkReply->close();

            QString fileTemplate = tempFile->fileTemplate();
            tempFile->deleteLater();
            tempFile = new QTemporaryFile(fileTemplate);

            QNetworkRequest request;
            request.setUrl(_url);
            request.setRawHeader("User-Agent", "Mozilla/5.0 (compatible; Konqueror/4.2; Linux) KHTML/4.1.4 (like Gecko) SUSE");
            QNetworkReply* reply = this->_networkAccessManager->get(request);
            connect(reply, SIGNAL(readyRead()), this, SLOT(dataHandler()));
            connect(reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(changeProgress(qint64, qint64)));
            _currentNetworkReply = reply;
        }
        else
        {
            emit error("Too many redirects");
        }
    }
    else if (networkReply->error())
    {
        emit error(networkReply->errorString(), this);
        networkReply->close();
        this->processNetworkReply();
    }
    else
    {
        this->processNetworkReply();
        networkReply->close();
    }
    networkReply->deleteLater();

}

void video::processNetworkReply()
{
    switch (this->_step)
    {
        case 1:
        case 2:
        {
            tempFile->close();
            tempFile->open();
            QByteArray data = tempFile->readAll();
            tempFile->close();
            tempFile->deleteLater();
            QString html = QString::fromUtf8(data, data.size());
            parseVideo(html);
            break;
        }
        case 3:
        {
            emit downloadFinished();
            break;
        }
    }
}

void video::parseVideo(QString)
{
    emit error("Video format not implemented");
}

void video::changeProgress(qint64 bytesReceived, qint64 bytesTotal)
{
    if (this->_treeItem)
    {
        if (bytesTotal > 0)
        {
            _downloadProgress = bytesReceived;
            _downloadSize = bytesTotal;
            this->_progressBar->setMaximum(bytesTotal);
            this->_progressBar->setValue(bytesReceived);
            for (int i = 0; i <= 3; i++)
            {
                this->_treeItem->setToolTip(i, "<strong>" + tr("Downloading ...") + "</strong><br />" + QString::number((double)bytesReceived/1024/1024, QLocale::system().decimalPoint().toAscii(), 1) + tr(" MiB") + "/" + QString::number((double)bytesTotal/1024/1024, QLocale::system().decimalPoint().toAscii(), 1) + tr(" MiB"));
            }
        }
    }
    else
    {
        qDebug() << bytesReceived << "of" << bytesTotal;
    }
    emit progressChanged(bytesReceived, bytesTotal);
}

void video::setQuality(int quality)
{
    this->_quality = quality;
}

QString video::quality()
{
    return this->_supportedQualities.at(_quality).quality;
}

video* video::createNewInstance()
{
    return new video();
}

QList<QString> video::getSupportedQualities()
{
    QStringList result;
    for (int i = 0; i < _supportedQualities.size(); ++i)
    {
        result << _supportedQualities.at(i).quality;
    }
    return result;
}

QString video::title()
{
    return _title;
}

void video::download()
{
    _step = 3;
    _redirectLevel = 0;
    tempFile = new QTemporaryFile(QDir::tempPath() + "/clipgrab-download--XXXXXX");
    QNetworkRequest request;
    request.setUrl(QUrl(this->_supportedQualities.at(_quality).videoUrl));
    request.setRawHeader("User-Agent", "Mozilla/5.0 (compatible; Konqueror/4.4; Linux) KHTML/4.4.0 (like Gecko) SUSE");
    request.setRawHeader( "Accept", "application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5");
    request.setRawHeader( "Accept-Charset", "ISO-8859-1,utf-8;q=0.7,*;q=0.3" );
    request.setRawHeader( "Accept-Encoding", "gzip,deflate,sdch" );
    request.setRawHeader( "Connection", "close" );
    qDebug() << "Starting download of " << this->_supportedQualities.at(_quality).videoUrl << " from " << this->_name;
    QNetworkReply* reply = this->_networkAccessManager->get(request);
    _networkAccessManager->disconnect();

    this->_redirectLevel = 0;
    _currentNetworkReply = reply;
    _downloadProgress = 0;
    _downloadSize = 1;
    connect(this->_networkAccessManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(handleNetworkReply(QNetworkReply*)));
    connect(reply, SIGNAL(readyRead()), this, SLOT(dataHandler()));
    connect(reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(changeProgress(qint64, qint64)));

}

void video::setTreeItem(QTreeWidgetItem* item)
{
    this->_treeItem = item;
}

void video::setFormat(int format)
{
    this->_format = format;
}

void video::startConvert()
{
    this->_progressBar->setMinimum(0);
    this->_progressBar->setMaximum(0);
    for (int i = 0; i <= 3; i++)
    {
        this->_treeItem->setToolTip(i, "<strong>" + tr("Converting ...") + "</strong>");
    }
    connect(this->_converter, SIGNAL(conversionFinished()), this, SLOT(conversionFinished()));
    this->_converter->startConversion(this->tempFile, _targetPath, _supportedQualities.at(_quality).containerName, _metaTitle, _metaArtist,_converterMode);

}

void video::setConverter(converter* converter, int mode)
{
    _converter = converter;
    _converterMode = mode;
}

void video::setTargetPath(QString target)
{
    _targetPath = target;
}

QString video::getSaveTitle()
{
    QString title = this->_title;
    if (title.size() > 0)
    {
        QStringList charsToBeRemoved;
        charsToBeRemoved << "\\" << "/" << ":" << "*" << "?" << "<" << ">" << "|" << "'" << "\"" << "&amp;quot;";
        title = title.simplified();

        if (title[0] == QChar('.'))
        {
            title = title.remove(0, 1);
        }
        if (title.size() > 0)
        {
            for (int i = 0; i < charsToBeRemoved.size(); ++i)
            {
                title = title.replace(charsToBeRemoved.at(i), "");
            }
        }
        else
        {
            return "download-empty-title";
        }
    }
    else
    {
        return "download-empty-title";
    }
    qDebug() << title;
    return title;
}

void video::conversionFinished()
{
    this->_progressBar->setMaximum(1);
    this->_progressBar->setValue(1);
    this->_progressBar->setFormat(tr("Finished"));
    this->_finished = true;
    for (int i = 0; i <= 3; i++)
    {
        this->_treeItem->setToolTip(i, "<strong>" + tr("Finished!") + "</strong>");
    }
    emit conversionFinished(this);

}

QString video::getSaveFileName()
{
    return _targetPath;
}

QString video::getTargetPath()
{
    return _targetPath;
}

QTreeWidgetItem* video::treeItem()
{
    return this->_treeItem;
}


bool video::isDownloadPaused()
{
    if (_currentNetworkReply)
    {
        if (_currentNetworkReply->readBufferSize() != 1)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
    else
    {
        return false;
    }
}

void video::togglePause()
{
    if (_currentNetworkReply)
    {
        if (_currentNetworkReply->readBufferSize() == 1)
        {
            _currentNetworkReply->setReadBufferSize(0);
        }
        else
        {
            _currentNetworkReply->setReadBufferSize(1);
        }
    }
}

QPair<qint64, qint64> video::downloadProgress()
{
    QPair<qint64, qint64> returnValue;
    returnValue.first = _downloadProgress;
    returnValue.second = _downloadSize;
    return returnValue;
}

void video::setMetaTitle(QString title)
{
    if (title.size() > 0)
    {
        QStringList charsToBeRemoved;
        charsToBeRemoved << "\\" << "/" << ":" << "*" << "?" << "<" << ">" << "|" << "'" << "\"" << "&amp;quot;";
        title = title.simplified();

        if (title[0] == QChar('.'))
        {
            title = title.remove(0, 1);
        }
        if (title.size() > 0)
        {
            for (int i = 0; i < charsToBeRemoved.size(); ++i)
            {
                title = title.replace(charsToBeRemoved.at(i), "");
            }
        }
        _metaTitle = title;
    }
}

void video::setMetaArtist(QString artist)
{
    if (artist.size() > 0)
    {
        QStringList charsToBeRemoved;
        charsToBeRemoved << "\\" << "!" << "\"";
        artist = artist.simplified();

        if (artist[0] == QChar('.'))
        {
            artist = artist.remove(0, 1);
        }
        if (artist.size() > 0)
        {
            for (int i = 0; i < charsToBeRemoved.size(); ++i)
            {
                artist = artist.replace(charsToBeRemoved.at(i), "");
            }
        }
        _metaArtist = artist;
    }
}

QString video::metaTitle()
{
    return _metaTitle;
}

QString video::metaArtist()
{
    return _metaArtist;
}

void video::dataHandler()
{
    if (tempFile->open())
    {
        tempFile->write(_currentNetworkReply->readAll());
    }
}

void video::slotAnalysingFinished()
{

    qDebug() << "Discovered: " << _title;
    for (int i = 0; i > _supportedQualities.size(); i++)
    {
        qDebug() << _supportedQualities.at(i).quality << _supportedQualities.at(i).containerName << _supportedQualities.at(i).videoUrl;
    }
}
